@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -32,6 +32,7 @@ import ch.kuramo.javie.api.IAnimatableLayerReference;
 import ch.kuramo.javie.api.IAnimatableValue;
 import ch.kuramo.javie.api.IAudioBuffer;
 import ch.kuramo.javie.api.IVideoBuffer;
+import ch.kuramo.javie.api.Quality;
 import ch.kuramo.javie.api.Resolution;
 import ch.kuramo.javie.api.Time;
 import ch.kuramo.javie.api.VideoBounds;
@@ -39,8 +40,9 @@ import ch.kuramo.javie.api.VideoBounds;
 /**
  * <p><code>IEffectContext</code> サービスは、エフェクトの実行に必要な情報
  * (エフェクトコンテキスト) とそれに関連したいくつかの処理を提供します。
- * エフェクトコンテキストには、現在時刻、レンダリング解像度 ({@link Resolution})、色深度
- * ({@link ColorMode})、{@link AudioMode}、オーディオのサンプルフレーム数が含まれます。
+ * エフェクトコンテキストには、現在時刻、品質スイッチの状態 ({@link Quality})、
+ * レンダリング解像度 ({@link Resolution})、色深度 ({@link ColorMode})、
+ * {@link AudioMode}、オーディオのサンプルフレーム数が含まれます。
  */
 public interface IEffectContext {
 
@@ -65,6 +67,15 @@ public interface IEffectContext {
 	 */
 	void setTime(Time time);
 
+	/**
+	 * 品質スイッチの状態を返します。
+	 * エフェクトは品質スイッチの状態によって処理内容を変えることができます。
+	 * 
+	 * @return 品質スイッチの状態
+	 * @since 0.5.1
+	 */
+	Quality getQuality();
+
 	/**
 	 * エフェクトコンテキストのレンダリング解像度を返します。
 	 * ビデオエフェクトはこのレンダリング解像度に従って適切な処理を行う必要があります。
