@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -55,6 +55,8 @@ public final class CommandIds {
 
 	public static final String WAVE_OUTPUT = "ch.kuramo.javie.app.commands.waveOutput";
 
+	public static final String BATCH_OUTPUT = "ch.kuramo.javie.app.commands.batchOutput";
+
 
 	public static final String UNDO = IWorkbenchCommandConstants.EDIT_UNDO;
 
@@ -79,6 +81,13 @@ public final class CommandIds {
 	public static final String SHOW_CONSOLE_VIEW = "ch.kuramo.javie.app.commands.showConsoleView";
 
 
+	public static final String MMD2JAVIE = "ch.kuramo.mmd2javie.commands.mmd2javie";
+
+	public static final String JAVIE2MMD = "ch.kuramo.javie2mmd.commands.javie2mmd";
+
+	public static final String VOODOO2JAVIE = "ch.kuramo.voodoo2javie.commands.voodoo2javie";
+
+
 	private CommandIds() { }
 
 }
