@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -23,6 +23,10 @@
 
 package ch.kuramo.javie.app.actions;
 
+import java.io.File;
+
+import org.eclipse.swt.widgets.FileDialog;
+
 import ch.kuramo.javie.app.CommandIds;
 import ch.kuramo.javie.app.views.LayerCompositionView;
 import ch.kuramo.javie.core.output.JavaImageSequenceOutput;
@@ -39,6 +43,11 @@ public class SequenceOutputAction extends AbstractOutputAction<JavaImageSequence
 		//setImageDescriptor(Activator.getImageDescriptor("/icons/sequence_output.png"));
 	}
 
+	@Override
+	protected String getDefaultExtension() {
+		return null;
+	}
+
 	@Override
 	protected JavaImageSequenceOutput createOutput(Injector injector) {
 		JavaImageSequenceOutput output = injector.getInstance(JavaImageSequenceOutput.class);
@@ -47,14 +56,10 @@ public class SequenceOutputAction extends AbstractOutputAction<JavaImageSequence
 	}
 
 	@Override
-	protected String getDefaultExtension() {
-		return null;
-	}
-
-	@Override
-	protected boolean isPromptedForFileOverwrite() {
-		// ファイルダイアログで指定するのは連番ファイルのプレフィックスなので、上書きの警告を出しても無意味。
-		return false;
+	protected FileDialog createFileDialog(File folder, String fileName) {
+		FileDialog dialog = super.createFileDialog(folder, fileName);
+		dialog.setOverwrite(false);		// ファイルダイアログで指定するのは連番ファイルのプレフィックスなので、上書きの警告を出しても無意味。
+		return dialog;
 	}
 
 	@Override
