@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Yoshikazu Kuramochi
+ * Copyright (c) 2009,2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -75,7 +75,7 @@ public class RemoveEffectsOperation extends ProjectOperation {
 
 
 	public RemoveEffectsOperation(ProjectManager pm, Collection<Object[]> effects) {
-		super(pm, "エフェクトの消去");
+		super(pm, "エフェクトの削除");
 		InjectorHolder.getInjector().injectMembers(this);
 
 		LayerComposition comp = pm.checkLayer((EffectableLayer) effects.iterator().next()[0]);
