@@ -367,7 +367,7 @@ public class ResetPropertiesOperation extends ProjectOperation {
 		Context jsContext = jsContextFactory.enterContext();
 		try {
 			Scriptable globalScope = jsContext.initStandardObjects();
-			ScriptableObject.putProperty(globalScope, "videoBounds", Context.javaToJS(videoBounds, globalScope));
+			ScriptableObject.putConstProperty(globalScope, "videoBounds", Context.javaToJS(videoBounds, globalScope));
 			Object jsValue = jsContext.evaluateString(globalScope, resetScript, propDesc.getName(), 1, null);
 
 			@SuppressWarnings("unchecked")
