@@ -47,7 +47,7 @@ public class AnimatableLayerReferenceElementDelegate extends AnimatableValueElem
 	}
 
 	protected void drawValue(GC gc, int x, int y, int height, boolean focused) {
-		drawValue(gc, x, y, height, focused, getName(getReferencedLayer()));
+		drawValue(gc, x, y, height, focused, getName(getLayerComposition().getLayer(valueToDraw)));
 	}
 
 	protected boolean canDragGestureEdit() {
@@ -58,11 +58,16 @@ public class AnimatableLayerReferenceElementDelegate extends AnimatableValueElem
 		throw new UnsupportedOperationException("drag gesture edit is not supported");
 	}
 
+	protected boolean isInPlaceEditorActive() {
+		// FIXME メニューが表示されているときは true を返すべき。
+		return false;
+	}
+
 	protected void openInPlaceEditor() {
 		Rectangle r = valueArea.get(valueIndex);
 		Menu menu = element.createPopUpMenu(r.x-5, r.y+r.height+2);
 
-		final Layer currentLayer = getReferencedLayer();
+		final Layer currentLayer = getLayerComposition().getLayer(valueWithoutExpr);
 
 		Action action = new Action(getName(null), IAction.AS_RADIO_BUTTON) {
 			public void run() {
@@ -95,10 +100,6 @@ public class AnimatableLayerReferenceElementDelegate extends AnimatableValueElem
 		return (LayerComposition) element.viewer.getData(LayerCompositionView.LAYER_COMPOSITION);
 	}
 
-	private Layer getReferencedLayer() {
-		return getLayerComposition().getLayer(value);
-	}
-
 	private boolean isLayerNameMode() {
 		return Boolean.TRUE.equals(element.viewer.getData(LayerCompositionView.LAYER_NAME_MODE));
 	}
