@@ -35,6 +35,7 @@ import ch.kuramo.javie.core.AnimatableColor;
 import ch.kuramo.javie.core.AnimatableDouble;
 import ch.kuramo.javie.core.AnimatableEnum;
 import ch.kuramo.javie.core.AnimatableInteger;
+import ch.kuramo.javie.core.AnimatableLayerReference;
 import ch.kuramo.javie.core.AnimatableString;
 import ch.kuramo.javie.core.AnimatableValue;
 import ch.kuramo.javie.core.AnimatableVec2d;
@@ -98,12 +99,25 @@ public class EffectAnimatableValueElement extends AnimatableValueElement {
 					increment, displayPrecision, editorPrecision, unit);
 
 		} else if (avalue instanceof AnimatableVec2d) {
-			return new AnimatableVec2dElementDelegate(this, label, (AnimatableVec2d) avalue,
-					increment, displayPrecision, editorPrecision, unit);
+			if (Boolean.valueOf(descriptor.getString("tiny2DManipulator"))) {
+				return new Tiny2DManipulatorElementDelegate(this, label, (AnimatableVec2d) avalue,
+						increment, displayPrecision, editorPrecision, unit);
+			} else if (Boolean.valueOf(descriptor.getString("xyLinkable"))) {
+				return new XYLinkableVec2dElementDelegate(this, label, (AnimatableVec2d) avalue,
+						increment, displayPrecision, editorPrecision, unit);
+			} else {
+				return new AnimatableVec2dElementDelegate(this, label, (AnimatableVec2d) avalue,
+						increment, displayPrecision, editorPrecision, unit);
+			}
 
 		} else if (avalue instanceof AnimatableDouble) {
-			return new AnimatableDoubleElementDelegate(this, label, (AnimatableDouble) avalue,
-					increment, displayPrecision, editorPrecision, unit);
+			if (Boolean.valueOf(descriptor.getString("angleControl"))) {
+				return new AngleElementDelegate(this, label, (AnimatableDouble) avalue,
+						increment, displayPrecision, editorPrecision, unit);
+			} else {
+				return new AnimatableDoubleElementDelegate(this, label, (AnimatableDouble) avalue,
+						increment, displayPrecision, editorPrecision, unit);
+			}
 
 		} else if (avalue instanceof AnimatableInteger) {
 			return new AnimatableIntegerElementDelegate(this, label, (AnimatableInteger) avalue,
@@ -123,6 +137,9 @@ public class EffectAnimatableValueElement extends AnimatableValueElement {
 			AnimatableEnumElementDelegate<?> delegate = new AnimatableEnumElementDelegate(this, label, (AnimatableEnum) avalue);
 			return delegate;
 
+		} else if (avalue instanceof AnimatableLayerReference) {
+			return new AnimatableLayerReferenceElementDelegate(this, label, (AnimatableLayerReference) avalue);
+
 		} else {
 			throw new JavieRuntimeException("unsupported property type: " + avalue.getClass().getName());
 		}
