@@ -25,6 +25,7 @@ package ch.kuramo.javie.app.views.layercomp;
 
 import java.util.List;
 import java.util.Map;
+import java.util.SortedMap;
 
 import org.eclipse.jface.action.Action;
 import org.eclipse.jface.action.ActionContributionItem;
@@ -48,6 +49,7 @@ import org.eclipse.ui.ISharedImages;
 import org.eclipse.ui.PlatformUI;
 
 import ch.kuramo.javie.api.BlendMode;
+import ch.kuramo.javie.api.Time;
 import ch.kuramo.javie.app.EnumLabels;
 import ch.kuramo.javie.app.ImageUtil;
 import ch.kuramo.javie.app.project.ModifyLayerBlendModeOperation;
@@ -57,10 +59,12 @@ import ch.kuramo.javie.app.project.ProjectManager;
 import ch.kuramo.javie.app.project.RenameLayerOperation;
 import ch.kuramo.javie.app.views.LayerCompositionView;
 import ch.kuramo.javie.app.widgets.WSWin32;
+import ch.kuramo.javie.core.AnimatableString;
 import ch.kuramo.javie.core.CameraLayer;
 import ch.kuramo.javie.core.EffectableLayer;
 import ch.kuramo.javie.core.ItemLayer;
 import ch.kuramo.javie.core.JavieRuntimeException;
+import ch.kuramo.javie.core.Keyframe;
 import ch.kuramo.javie.core.Layer;
 import ch.kuramo.javie.core.LayerComposition;
 import ch.kuramo.javie.core.LayerNature;
@@ -212,6 +216,19 @@ public class LayerElement extends Element {
 					} else {
 						name = itemName;
 					}
+				} else if (layer instanceof TextLayer) {
+					if (isLayerNameMode()) {
+						name = layer.getName();
+					} else {
+						AnimatableString sourceText = ((TextLayer) layer).getSourceText();
+						if (sourceText.hasKeyframe()) {
+							SortedMap<Time, Keyframe<String>> map = sourceText.getKeyframeMap();
+							name = map.get(map.firstKey()).value;
+						} else {
+							name = sourceText.getStaticValue();
+						}
+						name = name.replaceAll("\\s+", " ");
+					}
 				} else {
 					name = layer.getName();
 				}
