@@ -84,7 +84,7 @@ class TextSourceElementDelegate extends AnimatableStringElementDelegate {
 		final Text text = new Text(shell, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER);
 		text.setTabs(4);
 
-		String str = LF.matcher(value).replaceAll(Text.DELIMITER);
+		String str = LF.matcher(valueWithoutExpr).replaceAll(Text.DELIMITER);
 		text.setText(str);
 		text.selectAll();
 
