@@ -32,11 +32,14 @@ import java.util.SortedMap;
 import java.util.SortedSet;
 import java.util.TreeSet;
 
+import org.eclipse.core.commands.operations.IOperationHistory;
+import org.eclipse.core.commands.operations.IUndoableOperation;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.MouseAdapter;
 import org.eclipse.swt.events.MouseEvent;
 import org.eclipse.swt.events.PaintEvent;
 import org.eclipse.swt.events.PaintListener;
+import org.eclipse.swt.graphics.Color;
 import org.eclipse.swt.graphics.Cursor;
 import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.graphics.Image;
@@ -52,12 +55,11 @@ import org.eclipse.swt.widgets.Slider;
 import org.eclipse.swt.widgets.Tree;
 import org.eclipse.swt.widgets.TreeColumn;
 import org.eclipse.swt.widgets.TreeItem;
+import org.eclipse.ui.PlatformUI;
 
 import ch.kuramo.javie.api.Time;
 import ch.kuramo.javie.app.ColorUtil;
 import ch.kuramo.javie.app.ImageUtil;
-import ch.kuramo.javie.app.InjectorHolder;
-import ch.kuramo.javie.app.PropertyUtil;
 import ch.kuramo.javie.app.project.LayerSlipOperation;
 import ch.kuramo.javie.app.project.ModifyKeyframeInterpolationOperation;
 import ch.kuramo.javie.app.project.ModifyLayerInPointOperation;
@@ -69,8 +71,6 @@ import ch.kuramo.javie.app.project.ShiftKeyframesOperation;
 import ch.kuramo.javie.app.project.ShiftLayerTimesOperation;
 import ch.kuramo.javie.app.views.LayerCompositionView;
 import ch.kuramo.javie.core.AnimatableValue;
-import ch.kuramo.javie.core.Effect;
-import ch.kuramo.javie.core.EffectableLayer;
 import ch.kuramo.javie.core.Interpolation;
 import ch.kuramo.javie.core.JavieRuntimeException;
 import ch.kuramo.javie.core.Keyframe;
@@ -79,13 +79,8 @@ import ch.kuramo.javie.core.LayerComposition;
 import ch.kuramo.javie.core.LayerNature;
 import ch.kuramo.javie.core.MediaInput;
 import ch.kuramo.javie.core.MediaItemLayer;
-import ch.kuramo.javie.core.ProjectDecodeException;
 import ch.kuramo.javie.core.TimeCode;
 import ch.kuramo.javie.core.Util;
-import ch.kuramo.javie.core.services.ProjectDecoder;
-import ch.kuramo.javie.core.services.ProjectEncoder;
-
-import com.google.inject.Inject;
 
 public class TimelineManager {
 
@@ -129,20 +124,12 @@ public class TimelineManager {
 
 	private AbstractDragGestureEditor dragGestureEditor;
 
-	@Inject
-	private ProjectEncoder _encoder;
-
-	@Inject
-	private ProjectDecoder _decoder;
-
 
 	public TimelineManager(
 			ProjectManager projectManager, LayerComposition composition, LayerCompositionView view,
 			Composite meter, Scale zoomScale, Slider scrollSlider, Tree tree) {
 
 		super();
-		InjectorHolder.getInjector().injectMembers(this);
-
 		this.projectManager = projectManager;
 		this.composition = composition;
 		this.view = view;
@@ -292,13 +279,20 @@ public class TimelineManager {
 		Time frameDuration = composition.getFrameDuration();
 		int scroll = scrollSlider.getSelection();
 
-		int x1 = timeToPixels(currentTime, frameDuration, zoom) - scroll;
+		int x1 = timeToPixels(currentTime, frameDuration, zoom) - scroll + 1;
 		int x2 = timeToPixels(currentTime.add(frameDuration), frameDuration, zoom) - scroll;
+		int width = x2 - x1;
 
-		e.gc.setBackground(e.display.getSystemColor(SWT.COLOR_BLUE));
-		e.gc.setAlpha(48);
-		e.gc.fillRectangle(e.x+x1, e.y, x2-x1, e.height);
-		e.gc.setAlpha(255);
+		if (width >= 4) {
+			e.gc.setBackground(e.display.getSystemColor(SWT.COLOR_BLUE));
+			e.gc.setAlpha(48);
+			e.gc.fillRectangle(e.x+x1, e.y, width, e.height);
+			if (COCOA) {
+				e.gc.setAlpha(255);
+			} else {
+				e.gc.setAdvanced(false);
+			}
+		}
 	}
 
 	public void drawTimeIndicatorLine(PaintEvent e) {
@@ -471,7 +465,14 @@ public class TimelineManager {
 		int h = e.height-1;
 		GC gc = e.gc;
 
-		boolean hasSlipBar = false;
+		Time inPoint = layer.getInPoint();
+		Time outPoint = layer.getOutPoint();
+
+		int x1 = x + timeToPixels(inPoint, frameDuration, zoom) - scroll;
+		int x2 = x + timeToPixels(outPoint, frameDuration, zoom) - scroll;
+
+		gc.setBackground(ColorUtil.layerBar());
+		gc.fillRectangle(x1, y, x2-x1, h);
 
 		if (layer instanceof MediaItemLayer && !LayerNature.isTimeRemapEnabled(layer)) {
 			MediaInput input = ((MediaItemLayer) layer).getMediaInput();
@@ -481,35 +482,28 @@ public class TimelineManager {
 					Time startTime = layer.getStartTime();
 					Time endTime = startTime.add(new Time((long)(duration.timeValue/Math.abs(layer.getRate())), duration.timeScale));
 
-					int x1 = x + timeToPixels(startTime, frameDuration, zoom) - scroll;
-					int x2 = x + timeToPixels(endTime, frameDuration, zoom) - scroll;
+					int x3 = x + timeToPixels(startTime, frameDuration, zoom) - scroll;
+					int x4 = x + timeToPixels(endTime, frameDuration, zoom) - scroll;
 
-					gc.setBackground(e.display.getSystemColor(SWT.COLOR_RED));
-					gc.setAlpha(64);
-					gc.fillRectangle(x1, y, x2-x1, h);
+					if (x1 > x3) {
+						gc.setBackground(ColorUtil.layerSlipBar());
+						gc.fillRectangle(x3, y, x1-x3, h);
+					} else if (x1 < x3) {
+						gc.setBackground(ColorUtil.layerExceeded());
+						gc.fillRectangle(x1, y, x3-x1, h);
+					}
 
-					hasSlipBar = true;
+					if (x4 > x2) {
+						gc.setBackground(ColorUtil.layerSlipBar());
+						gc.fillRectangle(x2, y, x4-x2, h);
+					} else if (x2 > x4) {
+						gc.setBackground(ColorUtil.layerExceeded());
+						gc.fillRectangle(x4, y, x2-x4, h);
+					}
 				}
 			}
 		}
 
-		Time inPoint = layer.getInPoint();
-		Time outPoint = layer.getOutPoint();
-
-		int x1 = x + timeToPixels(inPoint, frameDuration, zoom) - scroll;
-		int x2 = x + timeToPixels(outPoint, frameDuration, zoom) - scroll;
-
-		if (!hasSlipBar) {
-			gc.setBackground(e.display.getSystemColor(SWT.COLOR_RED));
-			gc.setAlpha(64);
-			gc.fillRectangle(x1, y, x2-x1, h);
-		}
-		gc.setBackground(e.display.getSystemColor(SWT.COLOR_DARK_RED));
-		gc.setAlpha(128);
-		gc.fillRectangle(x1, y, x2-x1, h);
-		gc.setAlpha(255);
-
-
 		drawCurrentFrameRegion(e);
 	}
 
@@ -618,14 +612,48 @@ public class TimelineManager {
 		int y = e.y + e.height/2 - 1;
 		GC gc = e.gc;
 
+		Color color = ColorUtil.keyframe(false);
+		Color colorSelected = ColorUtil.keyframe(true);
+		Color colorBorder = ColorUtil.keyframeBorder();
+		int[] polygon = new int[8];
+
 		for (Keyframe<?> kf : avalue.getKeyframeMap().subMap(fromTime, toTime).values()) {
 			int x = ex + timeToPixels(kf.time, frameDuration, zoom) - scroll;
-			Image icon = ImageUtil.getKeyframeIcon(kf.interpolation, keyframeSelection.containsKey(kf));
-			if (icon != null) {
-				gc.drawImage(icon, x-5, y-5);
-			} else {
-				gc.setForeground(e.display.getSystemColor(SWT.COLOR_BLACK));
-				gc.drawRectangle(x-4, y-4, 8, 8);
+
+			if (COCOA) {
+				Image icon = ImageUtil.getKeyframeIcon(kf.interpolation, keyframeSelection.containsKey(kf));
+				if (icon != null) {
+					gc.drawImage(icon, x-5, y-5);
+					continue;
+				}
+			}
+
+			gc.setForeground(colorBorder);
+			gc.setBackground(keyframeSelection.containsKey(kf) ? colorSelected : color);
+
+			switch (kf.interpolation) {
+				case HOLD:
+					gc.fillRectangle(x-3, y-3, 7, 7);
+					gc.drawRectangle(x-4, y-4, 8, 8);
+					break;
+
+				case LINEAR:
+					polygon[0] = x+1; polygon[1] = y-5;
+					polygon[2] = x+5; polygon[3] = y+1;
+					polygon[4] = x  ; polygon[5] = y+5;
+					polygon[6] = x-4; polygon[7] = y;
+					gc.fillPolygon(polygon);
+					polygon[0] = x  ; //polygon[1] = y-5;
+					/*polygon[2] = x+5;*/ polygon[3] = y;
+					//polygon[4] = x  ; polygon[5] = y+5;
+					polygon[6] = x-5; //polygon[7] = y;
+					gc.drawPolygon(polygon);
+					break;
+
+				case CATMULL_ROM:
+					gc.fillOval(x-3, y-3, 7, 7);
+					gc.drawOval(x-4, y-4, 8, 8);
+					break;
 			}
 		}
 
@@ -729,45 +757,33 @@ public class TimelineManager {
 		return data;
 	}
 
-	private String[] createBaseAnimatableValues(Object[][] keyframeData) {
-		Map<AnimatableValue<?>, AnimatableValue<?>> map = Util.newMap();
-		AnimatableValue<?>[] baseAvalues = new AnimatableValue[keyframeData.length];
+	private Object[][] createSelectedKeyframeData2() {
+		Map<AnimatableValueElement, Set<Keyframe<?>>> map = Util.newMap();
+		List<Object[]> data = Util.newList();
 
-		for (int i = 0, n = keyframeData.length; i < n; ++i) {
-			Layer layer = composition.getLayer((String) keyframeData[i][0]);
-			Integer effectIndex = (Integer) keyframeData[i][1];
-			String property = (String) keyframeData[i][2];
-			Keyframe<?> kf = (Keyframe<?>) keyframeData[i][3];
+		for (Map.Entry<Keyframe<?>, AnimatableValueElement> e : keyframeSelection.entrySet()) {
+			Keyframe<?> kf = e.getKey();
+			AnimatableValueElement element = e.getValue();
 
-			AnimatableValue<?> avalue;
-			if (effectIndex == -1) {
-				avalue = PropertyUtil.getProperty(layer, property);
-			} else {
-				Effect effect = ((EffectableLayer) layer).getEffects().get(effectIndex);
-				avalue = PropertyUtil.getProperty(effect, property);
-			}
+			Set<Keyframe<?>> keyframes = map.get(element);
+			if (keyframes == null) {
+				keyframes = Util.newSet();
+				map.put(element, keyframes);
 
-			AnimatableValue<?> baseAvalue = map.get(avalue);
-			if (baseAvalue == null) {
-				String s = _encoder.encodeElement(avalue);
-				try {
-					baseAvalue = _decoder.decodeElement(s, avalue.getClass());
-				} catch (ProjectDecodeException e) {
-					throw new JavieRuntimeException(e);
+				if (element instanceof LayerAnimatableValueElement) {
+					LayerAnimatableValueElement avalueElem = (LayerAnimatableValueElement) element;
+					data.add(new Object[] { avalueElem.layer, -1, avalueElem.getProperty(), keyframes });
+				} else {
+					EffectAnimatableValueElement avalueElem = (EffectAnimatableValueElement) element;
+					data.add(new Object[] { avalueElem.layer,
+											avalueElem.layer.getEffects().indexOf(avalueElem.effect),
+											avalueElem.descriptor.getName(), keyframes });
 				}
-				map.put(avalue, baseAvalue);
-				baseAvalues[i] = baseAvalue;
 			}
-			baseAvalue.removeKeyframe(kf.time);
+			keyframes.add(kf);
 		}
 
-		String[] encodedBaseAvalues = new String[baseAvalues.length];
-		for (int i = 0, n = baseAvalues.length; i < n; ++i) {
-			if (baseAvalues[i] != null) {
-				encodedBaseAvalues[i] = _encoder.encodeElement(baseAvalues[i]);
-			}
-		}
-		return encodedBaseAvalues;
+		return data.toArray(new Object[data.size()][]);
 	}
 
 	public void removeSelectedKeyframes() {
@@ -1129,17 +1145,11 @@ public class TimelineManager {
 
 		private final Object[][] layersAndStartTimes;
 
-		private final Object[][] selectedKeyframeData;
-
-		private final String[] baseAnimatableValues;
-
 
 		private DragGestureSlipEditor(MouseEvent event, Layer baseLayer, Time baseTime) {
 			super(event, baseTime, new TreeSet<Time>());
 			baseStartTime = baseLayer.getStartTime();
 			layersAndStartTimes = getLayersAndStartTimes();
-			selectedKeyframeData = createSelectedKeyframeData();
-			baseAnimatableValues = createBaseAnimatableValues(selectedKeyframeData);
 		}
 
 		private Object[][] getLayersAndStartTimes() {
@@ -1156,10 +1166,15 @@ public class TimelineManager {
 
 		@Override
 		protected ProjectOperation createOperation(Time time, Time prevTime, Time baseTime, String relation, boolean snap) {
+			IOperationHistory oph = PlatformUI.getWorkbench().getOperationSupport().getOperationHistory();
+			IUndoableOperation undoOp = oph.getUndoOperation(projectManager.getUndoContext());
+			boolean firstOp = undoOp == null || !(undoOp instanceof LayerSlipOperation)
+							|| !relation.equals(((LayerSlipOperation) undoOp).getRelation());
+
 			Time deltaTime = adjustDeltaTime(time.subtract(baseTime), false, baseStartTime);
 			return new LayerSlipOperation(
-					projectManager, composition,
-					layersAndStartTimes, selectedKeyframeData, baseAnimatableValues,
+					projectManager, composition, layersAndStartTimes,
+					firstOp ? createSelectedKeyframeData2() : null,
 					deltaTime, relation);
 		}
 
@@ -1197,23 +1212,22 @@ public class TimelineManager {
 
 		private final Keyframe<?> baseKeyframe;
 
-		private final Object[][] selectedKeyframeData;
-
-		private final String[] baseAnimatableValues;
-
 
 		private DragGestureKeyframeShifter(MouseEvent event, Keyframe<?> baseKeyframe) {
 			super(event, baseKeyframe.time, collectSnapTimes(true, true, false, true));
 			this.baseKeyframe = baseKeyframe;
-			selectedKeyframeData = createSelectedKeyframeData();
-			baseAnimatableValues = createBaseAnimatableValues(selectedKeyframeData);
 		}
 
 		@Override
 		protected ProjectOperation createOperation(Time time, Time prevTime, Time baseTime, String relation, boolean snap) {
+			IOperationHistory oph = PlatformUI.getWorkbench().getOperationSupport().getOperationHistory();
+			IUndoableOperation undoOp = oph.getUndoOperation(projectManager.getUndoContext());
+			boolean firstOp = undoOp == null || !(undoOp instanceof ShiftKeyframesOperation)
+							|| !relation.equals(((ShiftKeyframesOperation) undoOp).getRelation());
+
 			Time deltaTime = adjustDeltaTime(time.subtract(baseTime), snap, baseKeyframe.time);
 			return new ShiftKeyframesOperation(
-					projectManager, composition, selectedKeyframeData, baseAnimatableValues, deltaTime, relation);
+					projectManager, composition, firstOp ? createSelectedKeyframeData2() : null, deltaTime, relation);
 		}
 
 	}
