@@ -32,17 +32,13 @@ import javax.vecmath.AxisAngle4d;
 import javax.vecmath.Matrix4d;
 import javax.vecmath.Quat4d;
 
-import org.mozilla.javascript.Context;
-import org.mozilla.javascript.NativeArray;
 import org.mozilla.javascript.Scriptable;
-import org.mozilla.javascript.Wrapper;
 
 import ch.kuramo.javie.api.IAnimatableVec3d;
 import ch.kuramo.javie.api.Time;
-import ch.kuramo.javie.api.Vec2d;
 import ch.kuramo.javie.api.Vec3d;
-import ch.kuramo.javie.core.exprelems.Vec2dProperty;
-import ch.kuramo.javie.core.exprelems.Vec3dProperty;
+import ch.kuramo.javie.core.expression.ExpressionUtils;
+import ch.kuramo.javie.core.expression.Vec3dProperty;
 
 public class AnimatableVec3d extends ArithmeticalAnimatableValue<Vec3d> implements IAnimatableVec3d {
 
@@ -192,35 +188,14 @@ public class AnimatableVec3d extends ArithmeticalAnimatableValue<Vec3d> implemen
 	}
 
 	public Vec3d jsToJava(Object jsValue) {
-		if (jsValue instanceof Wrapper) {
-			Object javaObj = ((Wrapper) jsValue).unwrap();
+		Object o = ExpressionUtils.toDoubleOrDoubleArray(jsValue);
 
-			if (javaObj instanceof Vec3dProperty) {
-				javaObj = ((Vec3dProperty) javaObj).getValue();
-			} else if (javaObj instanceof Vec2dProperty) {
-				javaObj = ((Vec2dProperty) javaObj).getValue();
-			}
+		if (o == null) {
+			return Vec3d.ZERO;
+		}
 
-			if (javaObj instanceof Vec3d) {
-				return (Vec3d) javaObj;
-
-			} else if (javaObj instanceof Vec2d) {
-				Vec2d v2d = (Vec2d) javaObj;
-				return new Vec3d(v2d.x, v2d.y);
-
-			} else if (javaObj instanceof Object[]) {
-				Object[] value = (Object[]) javaObj;
-				double[] array = new double[3];
-				for (int i = 0, n = Math.min(3, value.length); i < n; ++i) {
-					array[i] = (Double) Context.jsToJava(value[i], double.class);
-				}
-				return valueOf(array);
-			}
-
-		} else if (jsValue instanceof NativeArray) {
-			// 配列表現を用いた場合 (例: [time*10, 200, 0])
-
-			double[] array = (double[]) Context.jsToJava(jsValue, double[].class);
+		if (o instanceof double[]) {
+			double[] array = (double[]) o;
 			if (array.length < 3) {
 				double[] tmp = array;
 				array = new double[3];
@@ -229,12 +204,9 @@ public class AnimatableVec3d extends ArithmeticalAnimatableValue<Vec3d> implemen
 				}
 			}
 			return valueOf(array);
-
-		} else if (jsValue == null) {
-			return Vec3d.ZERO;
 		}
 
-		double d = (Double) Context.jsToJava(jsValue, double.class);
+		double d = (Double) o;
 		return new Vec3d(d, d, d);
 	}
 
