@@ -24,6 +24,8 @@
 package ch.kuramo.javie.core;
 
 import ch.kuramo.javie.api.ColorMode;
+import ch.kuramo.javie.api.IAudioBuffer;
+import ch.kuramo.javie.api.IVideoBuffer;
 import ch.kuramo.javie.api.Size2i;
 import ch.kuramo.javie.api.Time;
 
@@ -57,8 +59,8 @@ public interface Composition extends Expressionee {
 
 	void prepareExpression(ExpressionScope scope);
 
-	VideoBuffer renderVideoFrame();
+	IVideoBuffer renderVideoFrame();
 
-	AudioBuffer renderAudioChunk();
+	IAudioBuffer renderAudioChunk();
 
 }
