@@ -23,31 +23,10 @@
 
 package ch.kuramo.javie.core;
 
-import java.util.Locale;
-import java.util.MissingResourceException;
-import java.util.ResourceBundle;
-
 public enum Interpolation {
 
 	HOLD,
 	LINEAR,
 	CATMULL_ROM;
 
-
-	private final String label = getLabel();
-
-	private String getLabel() {
-		try {
-			String bundleName = Interpolation.class.getPackage().getName() + ".messages";
-			ResourceBundle bundle = ResourceBundle.getBundle(bundleName, Locale.getDefault());
-			return bundle.getString(String.format("%s.%s", Interpolation.class.getSimpleName(), name()));
-		} catch (MissingResourceException e) {
-			return name();
-		}
-	}
-
-	public String label() {
-		return label;
-	}
-
 }
