@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -263,6 +263,8 @@ public class VideoBufferImpl implements IVideoBuffer {
 				return TextureWrapMode.CLAMP_TO_EDGE;
 			case GL2.GL_REPEAT:
 				return TextureWrapMode.REPEAT;
+			case GL2.GL_MIRRORED_REPEAT:
+				return TextureWrapMode.MIRRORED_REPEAT;
 			default:
 				throw new IllegalStateException("unsupported wrap mode: " + params[0]);
 		}
@@ -280,6 +282,9 @@ public class VideoBufferImpl implements IVideoBuffer {
 			case REPEAT:
 				param = GL2.GL_REPEAT;
 				break;
+			case MIRRORED_REPEAT:
+				param = GL2.GL_MIRRORED_REPEAT;
+				break;
 			default:
 				throw new UnsupportedOperationException("unsupported wrap mode: " + wrapMode.name());
 		}
