@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -124,7 +124,11 @@ class ArrayPool<T> {
 				return;
 			}
 		}
-		throw new IllegalArgumentException("array length does not match: " + length);
+
+		// 最大サイズを超えている場合はgetの際に警告を出しているので、何もせず無視。
+		if (length <= ARRAY_LENGTHS[ARRAY_LENGTHS.length-1]) {
+			throw new IllegalArgumentException("array length does not match: " + length);
+		}
 	}
 
 }
