@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -55,6 +55,9 @@ import ch.kuramo.javie.core.MediaItemLayer;
 import ch.kuramo.javie.core.NullLayer;
 import ch.kuramo.javie.core.Project;
 import ch.kuramo.javie.core.SolidColorItem;
+import ch.kuramo.javie.core.TARangeSelector;
+import ch.kuramo.javie.core.TAWigglySelector;
+import ch.kuramo.javie.core.TextAnimator;
 import ch.kuramo.javie.core.TextLayer;
 import ch.kuramo.javie.core.Util;
 import ch.kuramo.javie.core.annotations.ProjectElement;
@@ -69,6 +72,9 @@ import ch.kuramo.javie.core.internal.MediaItemLayerImpl;
 import ch.kuramo.javie.core.internal.NullLayerImpl;
 import ch.kuramo.javie.core.internal.ProjectImpl;
 import ch.kuramo.javie.core.internal.SolidColorItemImpl;
+import ch.kuramo.javie.core.internal.TARangeSelectorImpl;
+import ch.kuramo.javie.core.internal.TAWigglySelectorImpl;
+import ch.kuramo.javie.core.internal.TextAnimatorImpl;
 import ch.kuramo.javie.core.internal.TextLayerImpl;
 import ch.kuramo.javie.core.services.ProjectElementFactory;
 
@@ -240,4 +246,16 @@ public class ProjectElementFactoryImpl implements ProjectElementFactory {
 		return textLayer;
 	}
 
+	public TextAnimator newTextAnimator() {
+		return _injector.getInstance(TextAnimatorImpl.class);
+	}
+
+	public TARangeSelector newTARangeSelector() {
+		return _injector.getInstance(TARangeSelectorImpl.class);
+	}
+
+	public TAWigglySelector newTAWigglySelector() {
+		return _injector.getInstance(TAWigglySelectorImpl.class);
+	}
+
 }
