@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -27,10 +27,13 @@ import java.util.List;
 
 import ch.kuramo.javie.api.IAudioBuffer;
 import ch.kuramo.javie.core.Effect;
+import ch.kuramo.javie.core.MediaLayer;
 import ch.kuramo.javie.core.WrappedOperation;
 
 public interface AudioEffectPipeline {
 
-	IAudioBuffer doAudioEffects(List<Effect> effects, WrappedOperation<IAudioBuffer> inputOperation);
+	IAudioBuffer doAudioEffects(
+			MediaLayer layer, List<Effect> effects,
+			WrappedOperation<IAudioBuffer> inputOperation);
 
 }
