@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 Yoshikazu Kuramochi
+ * Copyright (c) 2009,2010 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -27,8 +27,6 @@ import org.scannotation.AnnotationDB;
 
 public interface ShaderRegistry {
 
-	void buildShaders(AnnotationDB db, ClassLoader cl) throws ShaderBuildException;
-
-	ShaderProgram getProgram(Class<?> clazz, String name);
+	void scanShaders(AnnotationDB db, ClassLoader cl) throws ShaderRegistryException;
 
 }
