@@ -23,6 +23,9 @@
 
 #pragma once
 
+#include "ForcibleARGB32.h"
+
+
 class InputBase
 {
 	friend class Callback;
@@ -40,9 +43,8 @@ protected:
 	virtual HRESULT ConnectSourceToGrabber(IBaseFilter* source, IBaseFilter* grabber) = 0;
 	virtual HRESULT BufferCB(void* buffer, long bufferSize) = 0;
 
-	IPin* GetPin(IBaseFilter* filter, PIN_DIRECTION pindir);
-	IPin* GetInPin(IBaseFilter* filter)		{ return GetPin(filter, PINDIR_INPUT); }
-	IPin* GetOutPin(IBaseFilter* filter)	{ return GetPin(filter, PINDIR_OUTPUT); }
+	HRESULT NextPin(IEnumPins* enumPins, PIN_DIRECTION pindir, IPin** pin);
+	HRESULT FirstPin(IBaseFilter* filter, PIN_DIRECTION pindir, IPin** pin);
 	HRESULT Connect(IBaseFilter* from, IBaseFilter* to);
 
 	IGraphBuilder* mGraphBuilder;
@@ -68,21 +70,23 @@ public:
 	virtual HRESULT InitWithFile(LPCWSTR file);
 	HRESULT FrameImageAtTime(LONGLONG time, void* buffer);
 
-	long GetWidth()		{ return mWidth; }
-	long GetHeight()	{ return mHeight; }
-
-	LONGLONG GetFrameCount()	{ return mFrameCount; }
+	long GetWidth()				{ return mWidth; }
+	long GetHeight()			{ return mHeight; }
+	LONGLONG GetFrameDuration()	{ return mFrameDuration; }
+	DWORD GetFourCC()			{ return mFourCC; }
 
 protected:
+	DWORD FourCC();
 	virtual void GetMediaType(CMediaType* mediaType);
 	virtual HRESULT ConnectSourceToGrabber(IBaseFilter* source, IBaseFilter* grabber);
 	virtual HRESULT BufferCB(void* buffer, long bufferSize);
 
-//	ForcibleARGB32* mForcibleARGB32;
+	ForcibleARGB32* mForcibleARGB32;
 
 	long mWidth;
 	long mHeight;
-	LONGLONG mFrameCount;
+	LONGLONG mFrameDuration;
+	DWORD mFourCC;
 
 	void* mBuffer;
 };
@@ -113,6 +117,7 @@ protected:
 
 	char* mBuffer;
 	long mRestSize;
+	long mCallbackCount;
 };
 
 class Callback : public CUnknown, public ISampleGrabberCB
