@@ -47,13 +47,13 @@ public class BrightnessAndContrast {
 
 	@ShaderSource
 	public static final String[] BRIGHTNESS_AND_CONTRAST = {
-		"uniform sampler2DRect texture;",
+		"uniform sampler2D texture;",
 		"uniform float brightness;",
 		"uniform float contrastSlope;",
 		"",
 		"void main(void)",
 		"{",
-		"	vec4 color = texture2DRect(texture, gl_TexCoord[0].st);",
+		"	vec4 color = texture2D(texture, gl_TexCoord[0].st);",
 		"	if (color.a != 0.0) {",
 		"		vec3 rgb = color.rgb/color.a;",
 		"		rgb = (rgb+brightness-0.5)*contrastSlope+0.5;",
