@@ -136,6 +136,7 @@ public class Circle {
 		this.arrayPools = arrayPools;
 		circleSamplerProgram = shaders.getProgram(Circle.class, "CIRCLE_SAMPLER");
 		invertSamplerProgram = shaders.getProgram(Circle.class, "INVERT_SAMPLER");
+		blendSupport.setProgramsClass(blendSupport.getPremultAndMatteClass(false));
 	}
 
 	private class CircleParams {
@@ -169,6 +170,9 @@ public class Circle {
 			Color color = context.value(Circle.this.color);
 			double a = color.a;
 			this.color = new Color(color.r*a, color.g*a, color.b*a, a);
+			// FIXME ここで乗算済みにしているのは不適切かもしれない。
+			// このエフェクト内ではColor.clearに渡すところとシェーダに渡すところがあるが、
+			// Color.clearにはストレート、シェーダには乗算済みで渡す必要がある。
 
 			double edgeRadius = 0;
 			double thickness = 0;
