@@ -45,6 +45,7 @@ import ch.kuramo.javie.api.annotations.Effect.Categories;
 import ch.kuramo.javie.api.services.IShaderRegistry;
 import ch.kuramo.javie.api.services.IVideoEffectContext;
 import ch.kuramo.javie.api.services.IVideoRenderSupport;
+import ch.kuramo.javie.effects.TexCoords;
 
 import com.google.inject.Inject;
 
@@ -122,25 +123,38 @@ public class HSLKey {
 		Smoothing smoothing = context.value(this.smoothing);
 		boolean maskOnly = context.value(this.maskOnly);
 
-		IShaderProgram program1 = maskOnly ? maskOnlyProgram : keyingProgram;
-		IShaderProgram program2 = (smoothing == Smoothing.LOW) ? smoothingLowProgram
-								: (smoothing == Smoothing.HIGH) ? smoothingHighProgram : null;
-
 		IVideoBuffer buffer;
 		try {
-			buffer = support.useShaderProgram(program1, prepareUniforms(), null, input);
+			IShaderProgram program = maskOnly ? maskOnlyProgram : keyingProgram;
+			buffer = support.useShaderProgram(program, prepareUniforms(), null, input);
 		} finally {
 			input.dispose();
 		}
 
-		if (program2 == null) {
-			return buffer;
+		IShaderProgram program;
+		float[] neib;
+		switch (smoothing) {
+			case LOW:
+				program = smoothingLowProgram;
+				neib = TexCoords.neighboringOffset8(bounds.width, bounds.height);
+				break;
+
+			case HIGH:
+				program = smoothingHighProgram;
+				neib = TexCoords.neighboringOffset24(bounds.width, bounds.height);
+				break;
+
+			default:
+				return buffer;
 		}
 
 		try {
 			Set<GLUniformData> uniforms = new HashSet<GLUniformData>();
 			uniforms.add(new GLUniformData("texture", 0));
-			return support.useShaderProgram(program2, uniforms, null, buffer);
+			uniforms.add(new GLUniformData("neib[0]", 2, FloatBuffer.wrap(neib)));
+
+			return support.useShaderProgram(program, uniforms, null, buffer);
+
 		} finally {
 			buffer.dispose();
 		}
@@ -150,7 +164,7 @@ public class HSLKey {
 		Set<GLUniformData> uniforms = new HashSet<GLUniformData>();
 		uniforms.add(new GLUniformData("texture", 0));
 
-		Color keyColor = context.value(this.keyColor).clamp();
+		Color keyColor = context.value(this.keyColor);
 		Vec2d similarityHue = context.value(this.similarityHue);
 		Vec2d similaritySat = context.value(this.similaritySat);
 		Vec2d similarityLuma = context.value(this.similarityLuma);
