@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010 Yoshikazu Kuramochi
+ * Copyright (c) 2010,2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -23,8 +23,6 @@
 
 package ch.kuramo.javie.effects.simulation;
 
-import java.nio.ByteBuffer;
-import java.nio.ByteOrder;
 import java.nio.FloatBuffer;
 import java.util.HashSet;
 import java.util.Set;
@@ -53,8 +51,10 @@ import ch.kuramo.javie.api.annotations.Effect.Categories;
 import ch.kuramo.javie.api.services.IAntiAliasSupport;
 import ch.kuramo.javie.api.services.IArrayPools;
 import ch.kuramo.javie.api.services.IShaderRegistry;
+import ch.kuramo.javie.api.services.IVBOCache;
 import ch.kuramo.javie.api.services.IVideoEffectContext;
 import ch.kuramo.javie.api.services.IVideoRenderSupport;
+import ch.kuramo.javie.api.services.IVBOCache.VBOCacheRecord;
 
 import com.google.inject.Inject;
 
@@ -226,17 +226,21 @@ public class Shatter {
 
 	private final IArrayPools arrayPools;
 
+	private final IVBOCache vboCache;
+
 	private final IShaderProgram program;
 
 	@Inject
 	public Shatter(
 			IVideoEffectContext context, IVideoRenderSupport support,
-			IAntiAliasSupport aaSupport, IArrayPools arrayPools, IShaderRegistry shaders) {
+			IAntiAliasSupport aaSupport, IArrayPools arrayPools,
+			IVBOCache vboCache, IShaderRegistry shaders) {
 
 		this.context = context;
 		this.support = support;
 		this.aaSupport = aaSupport;
 		this.arrayPools = arrayPools;
+		this.vboCache = vboCache;
 		program = shaders.getProgram(Shatter.class, "SHATTER");
 	}
 
@@ -266,24 +270,24 @@ public class Shatter {
 		int i2 = (int)Math.ceil((b2[2]-(patOrigin.x+fragSize.x/2))/fragSize.x);
 		int j2 = (int)Math.ceil((b2[3]-(patOrigin.y+fragSize.y/2))/fragSize.y);
 
-		IArray<float[]> attribs = null;
+		int[] vboIdAndNumVerts;
 		int primitiveType;
 		try {
 			switch (context.value(pattern)) {
 				case RECTANGLE:
-					attribs = rectangle(fragSize, patOrigin, i1, j1, i2, j2);
+					vboIdAndNumVerts = rectangle(fragSize, patOrigin, i1, j1, i2, j2);
 					primitiveType = GL2.GL_QUADS;
 					break;
 				case BRICK:
-					attribs = brickOrHexagon(true, fragSize, patOrigin, i1, j1, i2, j2);
+					vboIdAndNumVerts = brickOrHexagon(true, fragSize, patOrigin, i1, j1, i2, j2);
 					primitiveType = GL2.GL_TRIANGLES;
 					break;
 				case HEXAGON:
-					attribs = brickOrHexagon(false, fragSize, patOrigin, i1, j1, i2, j2);
+					vboIdAndNumVerts = brickOrHexagon(false, fragSize, patOrigin, i1, j1, i2, j2);
 					primitiveType = GL2.GL_TRIANGLES;
 					break;
 				case OCTAGON_AND_DIAMOND:
-					attribs = octagonAndDiamond(fragSize, patOrigin, i1, j1, i2, j2);
+					vboIdAndNumVerts = octagonAndDiamond(fragSize, patOrigin, i1, j1, i2, j2);
 					primitiveType = GL2.GL_TRIANGLES;
 					break;
 				default:
@@ -293,15 +297,50 @@ public class Shatter {
 			// TODO ミップマップ／異方性フィルタ／スーパーサンプリング
 			input.setTextureFilter(TextureFilter.LINEAR);
 
-			return doShatter(input, attribs, primitiveType, patOrigin, patDirRad);
+			return doShatter(input, vboIdAndNumVerts[0], vboIdAndNumVerts[1], primitiveType, patOrigin, patDirRad);
 
 		} finally {
 			input.dispose();
-			if (attribs != null) attribs.release();
 		}
 	}
 
-	private IArray<float[]> rectangle(Vec2d size, Vec2d origin, int i1, int j1, int i2, int j2) {
+	private int createVBO(IArray<float[]> data, String token) {
+		GL2 gl = context.getGL().getGL2();
+		int[] vboId = new int[1];
+		try {
+			gl.glGenBuffers(1, vboId, 0);
+			gl.glBindBuffer(GL2.GL_ARRAY_BUFFER, vboId[0]);
+			gl.glBufferData(GL2.GL_ARRAY_BUFFER, data.getLength()*4,
+					FloatBuffer.wrap(data.getArray()), GL2.GL_STATIC_DRAW);
+
+			vboCache.put(this, new VBOCacheRecord(vboId[0], token));
+
+			int result = vboId[0];
+			vboId = null;
+			return result;
+
+		} finally {
+			gl.glBindBuffer(GL2.GL_ARRAY_BUFFER, 0);
+			if (vboId != null && vboId[0] != 0) gl.glDeleteBuffers(1, vboId, 0);
+		}
+	}
+
+	private int[] rectangle(Vec2d size, Vec2d origin, int i1, int j1, int i2, int j2) {
+
+		int numFragments = (i2-i1+1)*(j2-j1+1);
+		int numVertices = numFragments*4;		// 頂点数 ＝ 四角形の数 x 四角形1つあたりの頂点数(4)
+
+		String token = String.format("%s/%d,%d/%d,%d/%d,%d,%d,%d",
+				Pattern.RECTANGLE.name(),
+				Double.doubleToLongBits(size.x), Double.doubleToLongBits(size.y),
+				Double.doubleToLongBits(origin.x), Double.doubleToLongBits(origin.y),
+				i1, j1, i2, j2);
+
+		VBOCacheRecord rec = vboCache.get(this);
+		if (rec != null && token.equals(rec.data)) {
+			return new int[] { rec.vboId, numVertices };
+		}
+
 
 		double halfX = size.x/2;
 		double halfY = size.y/2;
@@ -314,33 +353,57 @@ public class Shatter {
 				{ (float)-halfX, (float) halfY }
 		};
 
-		int numOfFragments = (i2-i1+1)*(j2-j1+1);
+		IArray<float[]> data = null;
+		try {
+			// 配列の要素数 ＝ 頂点数 x 頂点あたりの座標数(4)
+			// 頂点あたりの座標数は、頂点の座標XとYの2つ、四角形の中心の座標XとYの計4つ
+			data = arrayPools.getFloatArray(numVertices*4);
+			float[] array = data.getArray();
 
-		// 配列の要素数 ＝ 四角形の数 x 四角形1つあたりの頂点数(4) x 頂点あたりの座標数(4)
-		// 頂点あたりの座標数は、頂点の座標XとYの2つ、四角形の中心の座標XとYの計4つ
-		IArray<float[]> data = arrayPools.getFloatArray(numOfFragments*4*4);
-		float[] array = data.getArray();
+			for (int j = j1, k = 0; j <= j2; ++j) {
+				for (int i = i1; i <= i2; ++i) {
+					float x = (float)(origin.x + size.x*i);
+					float y = (float)(origin.y + size.y*j);
 
-		for (int j = j1, k = 0; j <= j2; ++j) {
-			for (int i = i1; i <= i2; ++i) {
-				float x = (float)(origin.x + size.x*i);
-				float y = (float)(origin.y + size.y*j);
-
-				for (int h = 0; h < 4; ++h) {
-					array[k++] = pt[h][0];
-					array[k++] = pt[h][1];
-					array[k++] = x;
-					array[k++] = y;
+					for (int h = 0; h < 4; ++h) {
+						array[k++] = pt[h][0];
+						array[k++] = pt[h][1];
+						array[k++] = x;
+						array[k++] = y;
+					}
 				}
 			}
-		}
 
-		return data;
+			return new int[] { createVBO(data, token), numVertices };
+
+		} finally {
+			if (data != null) data.release();
+		}
 	}
 
-	private IArray<float[]> brickOrHexagon(boolean brick, Vec2d size, Vec2d origin, int i1, int j1, int i2, int j2) {
+	private int[] brickOrHexagon(boolean brick, Vec2d size, Vec2d origin, int i1, int j1, int i2, int j2) {
 
 		++i2;
+		if (!brick) {
+			--j1; 
+			++j2;
+		}
+
+		int numFragments = (i2-i1+1)*(j2-j1+1);
+		int numVertices = numFragments*6*3;		// 頂点数 ＝ 破片の数 x 破片1つあたりの頂点数(6) x 3(GL_TRIANGLESを使用するため)
+
+		String token = String.format("%s/%d,%d/%d,%d/%d,%d,%d,%d",
+				brick ? Pattern.BRICK.name() : Pattern.HEXAGON.name(),
+				Double.doubleToLongBits(size.x), Double.doubleToLongBits(size.y),
+				Double.doubleToLongBits(origin.x), Double.doubleToLongBits(origin.y),
+				i1, j1, i2, j2);
+
+		VBOCacheRecord rec = vboCache.get(this);
+		if (rec != null && token.equals(rec.data)) {
+			return new int[] { rec.vboId, numVertices };
+		}
+
+
 		double halfX = size.x/2;
 		float[][] pt;
 
@@ -358,9 +421,6 @@ public class Shatter {
 					{ (float)-halfX, (float)-halfY }
 			};
 		} else {
-			--j1; 
-			++j2;
-
 			double sizeY13 = size.y*1/3;
 			double sizeY23 = size.y*2/3;
 
@@ -376,48 +436,67 @@ public class Shatter {
 			};
 		}
 
-		int numOfFragments = (i2-i1+1)*(j2-j1+1);
+		IArray<float[]> data = null;
+		try {
+			// 配列の要素数 ＝ 頂点数 x 頂点あたりの座標数(4)
+			// 頂点あたりの座標数は、頂点の座標XとYの2つ、破片の中心の座標XとYの計4つ
+			data = arrayPools.getFloatArray(numVertices*4);
+			float[] array = data.getArray();
 
-		// 配列の要素数 ＝ 破片の数 x 破片1つあたりの頂点数(6) x 3(GL_TRIANGLESを使用するため) x 頂点あたりの座標数(4)
-		// 頂点あたりの座標数は、頂点の座標XとYの2つ、破片の中心の座標XとYの計4つ
-		IArray<float[]> data = arrayPools.getFloatArray(numOfFragments*6*3*4);
-		float[] array = data.getArray();
+			for (int j = j1, k = 0; j <= j2; ++j) {
+				double offsetX = (j%2) != 0 ? -halfX : 0;
 
-		for (int j = j1, k = 0; j <= j2; ++j) {
-			double offsetX = (j%2) != 0 ? -halfX : 0;
+				for (int i = i1; i <= i2; ++i) {
+					float x = (float)(origin.x + size.x*i + offsetX);
+					float y = (float)(origin.y + size.y*j);
 
-			for (int i = i1; i <= i2; ++i) {
-				float x = (float)(origin.x + size.x*i + offsetX);
-				float y = (float)(origin.y + size.y*j);
+					for (int h = 0; h < 6; ++h) {
+						array[k++] = 0;
+						array[k++] = 0;
+						array[k++] = x;
+						array[k++] = y;
 
-				for (int h = 0; h < 6; ++h) {
-					array[k++] = 0;
-					array[k++] = 0;
-					array[k++] = x;
-					array[k++] = y;
+						array[k++] = pt[h][0];
+						array[k++] = pt[h][1];
+						array[k++] = x;
+						array[k++] = y;
 
-					array[k++] = pt[h][0];
-					array[k++] = pt[h][1];
-					array[k++] = x;
-					array[k++] = y;
-
-					array[k++] = pt[h+1][0];
-					array[k++] = pt[h+1][1];
-					array[k++] = x;
-					array[k++] = y;
+						array[k++] = pt[h+1][0];
+						array[k++] = pt[h+1][1];
+						array[k++] = x;
+						array[k++] = y;
+					}
 				}
 			}
-		}
 
-		return data;
+			return new int[] { createVBO(data, token), numVertices };
+
+		} finally {
+			if (data != null) data.release();
+		}
 	}
 
-	private IArray<float[]> octagonAndDiamond(Vec2d size, Vec2d origin, int i1, int j1, int i2, int j2) {
+	private int[] octagonAndDiamond(Vec2d size, Vec2d origin, int i1, int j1, int i2, int j2) {
 		// 右端と下端それぞれに１つずつ増やす必要があるのは、厳密には菱形の方だけ。
 		// 処理を分けるのも面倒なので八角形も増やしてしまっている。
 		++i2;
 		++j2;
 
+		int numFragments = (i2-i1+1)*(j2-j1+1);
+		int numVertices = numFragments*(8+4)*3;		// 頂点数 ＝ 破片の数 x 破片1つあたりの頂点数(8および4) x 3(GL_TRIANGLESを使用するため)
+
+		String token = String.format("%s/%d,%d/%d,%d/%d,%d,%d,%d",
+				Pattern.OCTAGON_AND_DIAMOND.name(),
+				Double.doubleToLongBits(size.x), Double.doubleToLongBits(size.y),
+				Double.doubleToLongBits(origin.x), Double.doubleToLongBits(origin.y),
+				i1, j1, i2, j2);
+
+		VBOCacheRecord rec = vboCache.get(this);
+		if (rec != null && token.equals(rec.data)) {
+			return new int[] { rec.vboId, numVertices };
+		}
+
+
 		double theta = Math.atan2(size.y, size.x);
 		double cos = Math.cos(theta);
 		double sin = Math.sin(theta);
@@ -447,61 +526,65 @@ public class Shatter {
 				}
 		};
 
-		int numOfFragments = (i2-i1+1)*(j2-j1+1);
+		IArray<float[]> data = null;
+		try {
+			// 配列の要素数 ＝ 頂点数 x 頂点あたりの座標数(4)
+			// 頂点あたりの座標数は、頂点の座標XとYの2つ、破片の中心の座標XとYの計4つ
+			data = arrayPools.getFloatArray(numVertices*4);
+			float[] array = data.getArray();
 
-		// 配列の要素数 ＝ 破片の数 x 破片1つあたりの頂点数(8および4) x 3(GL_TRIANGLESを使用するため) x 頂点あたりの座標数(4)
-		// 頂点あたりの座標数は、頂点の座標XとYの2つ、破片の中心の座標XとYの計4つ
-		IArray<float[]> data = arrayPools.getFloatArray(numOfFragments*(8+4)*3*4);
-		float[] array = data.getArray();
+			double halfX = size.x/2;
+			double halfY = size.y/2;
 
-		double halfX = size.x/2;
-		double halfY = size.y/2;
+			for (int j = j1, k = 0; j <= j2; ++j) {
+				for (int i = i1; i <= i2; ++i) {
+					float x = (float)(origin.x + size.x*i);
+					float y = (float)(origin.y + size.y*j);
 
-		for (int j = j1, k = 0; j <= j2; ++j) {
-			for (int i = i1; i <= i2; ++i) {
-				float x = (float)(origin.x + size.x*i);
-				float y = (float)(origin.y + size.y*j);
+					for (int h = 0; h < 8; ++h) {
+						array[k++] = 0;
+						array[k++] = 0;
+						array[k++] = x;
+						array[k++] = y;
 
-				for (int h = 0; h < 8; ++h) {
-					array[k++] = 0;
-					array[k++] = 0;
-					array[k++] = x;
-					array[k++] = y;
+						array[k++] = pt[0][h][0];
+						array[k++] = pt[0][h][1];
+						array[k++] = x;
+						array[k++] = y;
 
-					array[k++] = pt[0][h][0];
-					array[k++] = pt[0][h][1];
-					array[k++] = x;
-					array[k++] = y;
+						array[k++] = pt[0][h+1][0];
+						array[k++] = pt[0][h+1][1];
+						array[k++] = x;
+						array[k++] = y;
+					}
 
-					array[k++] = pt[0][h+1][0];
-					array[k++] = pt[0][h+1][1];
-					array[k++] = x;
-					array[k++] = y;
-				}
+					x -= halfX;
+					y -= halfY;
 
-				x -= halfX;
-				y -= halfY;
+					for (int h = 0; h < 4; ++h) {
+						array[k++] = 0;
+						array[k++] = 0;
+						array[k++] = x;
+						array[k++] = y;
 
-				for (int h = 0; h < 4; ++h) {
-					array[k++] = 0;
-					array[k++] = 0;
-					array[k++] = x;
-					array[k++] = y;
+						array[k++] = pt[1][h][0];
+						array[k++] = pt[1][h][1];
+						array[k++] = x;
+						array[k++] = y;
 
-					array[k++] = pt[1][h][0];
-					array[k++] = pt[1][h][1];
-					array[k++] = x;
-					array[k++] = y;
-
-					array[k++] = pt[1][h+1][0];
-					array[k++] = pt[1][h+1][1];
-					array[k++] = x;
-					array[k++] = y;
+						array[k++] = pt[1][h+1][0];
+						array[k++] = pt[1][h+1][1];
+						array[k++] = x;
+						array[k++] = y;
+					}
 				}
 			}
-		}
 
-		return data;
+			return new int[] { createVBO(data, token), numVertices };
+
+		} finally {
+			if (data != null) data.release();
+		}
 	}
 
 	private double[] rotate(VideoBounds bounds, Vec2d origin, double rad) {
@@ -539,7 +622,8 @@ public class Shatter {
 	}
 
 	private IVideoBuffer doShatter(
-			IVideoBuffer input, final IArray<float[]> attribs, final int primitiveType, Vec2d patOrigin, double patDirRad) {
+			IVideoBuffer input, final int vboId, final int numVertices,
+			final int primitiveType, Vec2d patOrigin, double patDirRad) {
 
 		final VideoBounds bounds = input.getBounds();
 		final int[] viewport = new int[4];
@@ -559,34 +643,30 @@ public class Shatter {
 					gl.glMatrixMode(GL2.GL_MODELVIEW);
 					gl.glLoadMatrixd(mvMatrix, 0);
 
-					// TODO ダイレクトバッファも配列と同様にプールする仕組みが必要
-					ByteBuffer directAttribs = ByteBuffer.allocateDirect(attribs.getLength()*4);	// この4は float のバイト数
-					directAttribs.order(ByteOrder.nativeOrder());
-					directAttribs.asFloatBuffer().put(attribs.getArray(), 0, attribs.getLength());
-
 					int vertOffsetLoc = program.getAttributeLocation("vertOffset");
 					int fragCenterLoc = program.getAttributeLocation("fragCenter");
 					try {
 						gl.glEnableVertexAttribArray(vertOffsetLoc);
 						gl.glEnableVertexAttribArray(fragCenterLoc);
+						gl.glBindBuffer(GL2.GL_ARRAY_BUFFER, vboId);
 
-						// この8は、ひとつの頂点あたり配列要素2つ x float のバイト数 
-						directAttribs.position(8);
-						gl.glVertexAttribPointer(fragCenterLoc, 2, GL2.GL_FLOAT, false, 4*4, directAttribs.slice());
-												// ひとつの頂点あたり配列要素2つ　　　　　　　// ストライドの要素数 x floatのバイト数
+						gl.glVertexAttribPointer(fragCenterLoc, 2, GL2.GL_FLOAT, false, 4*4, 8);
+														// 2: ひとつの頂点あたり配列要素2つ
+														// 4*4: ストライドの要素数 x floatのバイト数
+														// 8: ひとつの頂点あたり配列要素2つ x float のバイト数
 	
-						directAttribs.position(0);
-						gl.glVertexAttribPointer(vertOffsetLoc, 2, GL2.GL_FLOAT, false, 4*4, directAttribs);
+						gl.glVertexAttribPointer(vertOffsetLoc, 2, GL2.GL_FLOAT, false, 4*4, 0);
 
 						aaSupport.antiAlias(bounds.width, bounds.height, true, Color.COLORLESS_TRANSPARENT, new Runnable() {
 							public void run() {
-								gl.glDrawArrays(primitiveType, 0, attribs.getLength()/4);	// この4はストライドの要素数
+								gl.glDrawArrays(primitiveType, 0, numVertices);
 							}
 						});
 
 					} finally {
 						gl.glDisableVertexAttribArray(vertOffsetLoc);
 						gl.glDisableVertexAttribArray(fragCenterLoc);
+						gl.glBindBuffer(GL2.GL_ARRAY_BUFFER, 0);
 					}
 				}
 			};
@@ -779,7 +859,7 @@ public class Shatter {
 //			++i;
 //		}
 
-		uniforms.add(new GLUniformData("numOfForces", i));
+//		uniforms.add(new GLUniformData("numOfForces", i));
 		uniforms.add(new GLUniformData("forceTime[0]", 1, FloatBuffer.wrap(forceTime)));
 		uniforms.add(new GLUniformData("forcePosition[0]", 2, FloatBuffer.wrap(forcePosition)));
 		uniforms.add(new GLUniformData("forceRadius[0]", 1, FloatBuffer.wrap(forceRadius)));
@@ -855,7 +935,7 @@ public class Shatter {
 		"uniform float rotationSpeed;",
 		"uniform vec3 gravity;",
 		"",
-		"uniform int numOfForces;",
+//		"uniform int numOfForces;",
 		"uniform float forceTime[3];",		// あらかじめ time プロパティから forceNTime プロパティを引いておいたもの
 		"uniform vec2 forcePosition[3];",
 		"uniform float forceRadius[3];",
@@ -875,14 +955,14 @@ public class Shatter {
 		"	float t = 0.0;",		// フォースが到達してからの経過時間
 		"	vec3 v = vec3(0.0);",	// フォース到達時の破片の初速
 		"",
-		"	for (int i = 0; i < numOfForces; ++i) {",
-		"		float ft  = forceTime[i];",
-		"		vec2  fp  = forcePosition[i];",
-		"		float fr  = forceRadius[i];",
-		"		float fdl = forceDelay[i];",
-		"		float fs  = forceStrength[i];",
-		"		float fdc = forceDecay[i];",
-		"		float fdp = forceDepth[i];",
+		"	/*for (int i = 0; i < numOfForces; ++i)*/ {",
+		"		float ft  = forceTime[/*i*/0];",
+		"		vec2  fp  = forcePosition[/*i*/0];",
+		"		float fr  = forceRadius[/*i*/0];",
+		"		float fdl = forceDelay[/*i*/0];",
+		"		float fs  = forceStrength[/*i*/0];",
+		"		float fdc = forceDecay[/*i*/0];",
+		"		float fdp = forceDepth[/*i*/0];",
 		"",
 		"		vec2 fragCenterFromFp = fc - fp;",				// forcePosition から破片の中心へのベクトル
 		"		float distance = length(fragCenterFromFp);",	// forcePosition から破片の中心までの距離
