@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -25,17 +25,20 @@ package ch.kuramo.javie.effects.transition;
 
 import javax.media.opengl.GL2;
 
+import ch.kuramo.javie.api.ColorMode;
 import ch.kuramo.javie.api.IAnimatableDouble;
 import ch.kuramo.javie.api.IArray;
 import ch.kuramo.javie.api.IVideoBuffer;
+import ch.kuramo.javie.api.Quality;
+import ch.kuramo.javie.api.Resolution;
 import ch.kuramo.javie.api.VideoBounds;
 import ch.kuramo.javie.api.annotations.Effect;
 import ch.kuramo.javie.api.annotations.Property;
 import ch.kuramo.javie.api.annotations.Effect.Categories;
 import ch.kuramo.javie.api.services.IArrayPools;
+import ch.kuramo.javie.api.services.ITexture1DSupport;
 import ch.kuramo.javie.api.services.IVideoEffectContext;
 import ch.kuramo.javie.api.services.IVideoRenderSupport;
-import ch.kuramo.javie.effects.Texture1D;
 
 import com.google.inject.Inject;
 
@@ -56,12 +59,18 @@ public class LinearWipe {
 
 	private final IVideoRenderSupport support;
 
+	private final ITexture1DSupport tex1DSupport;
+
 	private final IArrayPools arrayPools;
 
 	@Inject
-	public LinearWipe(IVideoEffectContext context, IVideoRenderSupport support, IArrayPools arrayPools) {
+	public LinearWipe(
+			IVideoEffectContext context, IVideoRenderSupport support,
+			ITexture1DSupport tex1DSupport, IArrayPools arrayPools) {
+
 		this.context = context;
 		this.support = support;
+		this.tex1DSupport = tex1DSupport;
 		this.arrayPools = arrayPools;
 	}
 
@@ -188,7 +197,15 @@ public class LinearWipe {
 			texCoords[0] = (float)((2+2*a-b)*(1-completion)+b*completion);
 			texCoords[1] = (float)((1-b)*(1-completion)-(1+2*a-b)*completion);
 
-			return Texture1D.fromArray(data, context);
+			Quality quality = context.getQuality();
+			Resolution resolution = context.getVideoResolution();
+			ColorMode colorMode = context.getColorMode();
+			int internalFormat = (quality == Quality.DRAFT || resolution.scale < 1) ? GL2.GL_ALPHA8
+								: colorMode == ColorMode.RGBA32_FLOAT ? GL2.GL_ALPHA32F
+								: colorMode == ColorMode.RGBA16_FLOAT ? GL2.GL_ALPHA16F
+								: quality == Quality.BEST ? GL2.GL_ALPHA16F : GL2.GL_ALPHA8;
+
+			return tex1DSupport.texture1DFromArray(data, GL2.GL_ALPHA, internalFormat);
 
 		} finally {
 			data.release();
